@@ -1,5 +1,7 @@
-use std::io::{self, Write};
-use std::time::{Duration, Instant};
+use std::{
+    io::{self, Write},
+    time::{Duration, Instant},
+};
 
 use crate::util::now;
 
